/**
 *
 * \file
 *
 * \brief WINC1500 AP Mode Example.
 *
 * Copyright (c) 2018 Atmel Corporation. All rights reserved.
 *
 * \asf_license_start
 *
 * \page License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * \asf_license_stop
 *
 */

#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "APP1632_LCM.h"
#include "atmel_start.h"
#include "stdio_start.h"
#include "hal_delay.h"
#include "socket/include/socket.h"
#include "main.h"
#include "winc_init.h"
#include "driver/source/nmasic.h"

void			PowerOn_Beep(void) ;
void			TIMER_0_Initial(void) ;
void			I2C_0_Initial(void) ;
void			AT24MAC402_WR_Test(void) ;
void			MCP9808_Read(void) ;
unsigned int	ADC_Read_sync(void) ;
void			Board_Test(void) ;
void			Connection_Beep(void) ;
void			DisConnection_Beep(void) ;

unsigned int	LED_ScanNumber = 1 ; 
unsigned int	Report_Temp = 0 ;
unsigned int	The_Counter = 0 ;

unsigned int	Buzzer_Action = 0 ;
unsigned int	Buzzer_ON = 0 ;
unsigned int	Buzzer_State = 0 ;
unsigned int	Buzzer_Timer = 0 ;
unsigned int	Buzzer_Index = 0 ;
unsigned int	Buzzer_Action_Data[10] ;		// On/Off time = 10ms * n 

unsigned int	Error_Code = 1 ;

unsigned int	LED_Speed = 1 ;
unsigned int	LED_Speed_Data[16] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16} ;
unsigned int	LED_Speed_Timer = 0 ;
unsigned int	VR1_Value ;

unsigned int TempValue ;

static struct timer_task TIMER_0_task1, TIMER_0_task2, TIMER_0_task3, TIMER_0_task4 ;
struct io_descriptor *I2C_0_io;

/**
 * Example of using TIMER_0.
 */
static void TIMER_0_task1_cb(const struct timer_task *const timer_task)
{
		if (Buzzer_ON)
		{
			gpio_toggle_pin_level(Buzzer) ;
		}
		
}

static void TIMER_0_task2_cb(const struct timer_task *const timer_task)
{
	
		if ( Buzzer_Action)
		{
			switch (Buzzer_State)
			{
				case 1: 
						Buzzer_Timer = Buzzer_Action_Data[Buzzer_Index]; 
						if (Buzzer_Timer == 999)
						{
							Buzzer_State = 9 ;
							break ;
						}
						Buzzer_ON = 1 ;
						Buzzer_State = 2 ;
						Buzzer_Index ++ ;
						break ;
				case 2:
						Buzzer_Timer -- ;
						if (Buzzer_Timer == 0)
						{
							Buzzer_ON = 0 ;
							Buzzer_State = 3 ;
						}
						break ;
				case 3 :
						Buzzer_Timer = Buzzer_Action_Data[Buzzer_Index];
						if (Buzzer_Timer == 999)
						{
							Buzzer_State = 9 ;
							break ;
						}
						Buzzer_ON = 0 ;
						Buzzer_State = 4 ;
						Buzzer_Index ++ ;
						break ;			
				case 4:	
						Buzzer_Timer -- ;
						if (Buzzer_Timer == 0)
							Buzzer_State = 1 ;
						break ;						

				case 9:
						Buzzer_ON = 0 ;
						Buzzer_Action = 0 ;
						Buzzer_Index = 0 ;
						Buzzer_Timer = 0 ;
						Buzzer_State = 0 ;
						break ;
				default :
						Buzzer_ON = 0 ;
						break ;
			}
		}

}

static void TIMER_0_task3_cb(const struct timer_task *const timer_task)
{
	
if (Error_Code ==0)
{
	LED_Speed_Timer ++ ;
	if ( LED_Speed_Timer >= LED_Speed )
 {
		
	LED_Speed_Timer = 0 ;
	
	switch(LED_ScanNumber)
	{
		case (1) :
				gpio_set_pin_level(LED1,false);
				gpio_set_pin_level(LED2,true);
				gpio_set_pin_level(LED3,true);
				gpio_set_pin_level(LED4,true);
				LED_ScanNumber = 2 ;
				break ;
		case (2) :
				gpio_set_pin_level(LED1,true);
				gpio_set_pin_level(LED2,false);
				gpio_set_pin_level(LED3,true);
				gpio_set_pin_level(LED4,true);		
				LED_ScanNumber = 3 ;
				break ;				
		case(3) :
				gpio_set_pin_level(LED1,true);
				gpio_set_pin_level(LED2,true);
				gpio_set_pin_level(LED3,false);
				gpio_set_pin_level(LED4,true);
				LED_ScanNumber = 4 ;
				break ;
		case(4) :
				gpio_set_pin_level(LED1,true);
				gpio_set_pin_level(LED2,true);
				gpio_set_pin_level(LED3,true);
				gpio_set_pin_level(LED4,false);
				LED_ScanNumber = 5 ;
				break ;
		case (5) :
				gpio_set_pin_level(LED1,true);
				gpio_set_pin_level(LED2,true);
				gpio_set_pin_level(LED3,false);
				gpio_set_pin_level(LED4,true);
				LED_ScanNumber = 6 ;
				break ;
		case (6) :
				gpio_set_pin_level(LED1,true);
				gpio_set_pin_level(LED2,false);
				gpio_set_pin_level(LED3,true);
				gpio_set_pin_level(LED4,true);		
				LED_ScanNumber = 1 ;
				break ;				
		default:
				gpio_set_pin_level(LED1,true);
				break ;
	}
 }
}
}

static void TIMER_0_task4_cb(const struct timer_task *const timer_task)
{
		Report_Temp = 1 ;
		switch (Error_Code)
		{
			case 1:
					gpio_toggle_pin_level(LED1) ;
					break ;
			case 2:
					gpio_toggle_pin_level(LED2) ;
					break ;
			case 3:
					gpio_toggle_pin_level(LED3) ;
					break ;
			case 4:
					gpio_toggle_pin_level(LED4) ;
					break ;
		}

}

void TIMER_0_Initial(void)
{
	TIMER_0_task1.interval = 2;
	TIMER_0_task1.cb       = TIMER_0_task1_cb;
	TIMER_0_task1.mode     = TIMER_TASK_REPEAT;
	TIMER_0_task2.interval = 100;						// 100us * 100 = 10 ms
	TIMER_0_task2.cb       = TIMER_0_task2_cb;
	TIMER_0_task2.mode     = TIMER_TASK_REPEAT;
	TIMER_0_task3.interval = 250;						// 100us * 1000 = 25 ms
	TIMER_0_task3.cb       = TIMER_0_task3_cb;
	TIMER_0_task3.mode     = TIMER_TASK_REPEAT;	
	TIMER_0_task4.interval = 5000;						// 100us * 5000 = 500 ms
	TIMER_0_task4.cb       = TIMER_0_task4_cb;
	TIMER_0_task4.mode     = TIMER_TASK_REPEAT;		

	timer_add_task(&TIMER_0, &TIMER_0_task1);
	timer_add_task(&TIMER_0, &TIMER_0_task2);
	timer_add_task(&TIMER_0, &TIMER_0_task3);
	timer_add_task(&TIMER_0, &TIMER_0_task4);
	timer_start(&TIMER_0);
}

void I2C_0_Initial(void)
{

	i2c_m_sync_get_io_descriptor(&I2C_0, &I2C_0_io);
	i2c_m_sync_enable(&I2C_0);

}


void MCP9808_Read()
{
	uint8_t readBuf[2];
	uint8_t WORD_address[1] = { 0x07 };


	// Set I2C address to MCP9808 , A2/A1/A0 = 111, address = 0011 111 = 0x1F
	i2c_m_sync_set_slaveaddr(&I2C_0, 0x1F, I2C_M_SEVEN);

	// Random Read DATA_bytes
	memset( readBuf, 0, sizeof(readBuf) );
	// Write Temperature Read Command : Address 0110
	WORD_address[0] = 0x05 ;
	io_write(I2C_0_io, WORD_address, 1);
	// Sequential Read
	io_read(I2C_0_io, readBuf, 2);
	
	readBuf[0] &= 0x0f ;
	TempValue = ( (unsigned int)readBuf[0] * 256 + readBuf[1] ) * 0.0625 ;

}


void AT24MAC402_WR_Test()
{
	uint8_t readBuf[16];
	uint8_t WORD_address[1] = { 0x0 };
	uint8_t DATA_bytes[17] = { 0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8,
	0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF, 0x10 };

	// Set I2C address to AT24MAC402 , A2/A1/A0 = 110, address = 1010 110 = 0x56
	i2c_m_sync_set_slaveaddr(&I2C_0, 0x56, I2C_M_SEVEN);

	// Page Write DATA_bytes, first byte is WORD_address
	io_write(I2C_0_io, DATA_bytes, 17);

	// Write Cycle Time >5ms
	delay_ms(5);

	// Random Read DATA_bytes
	memset( readBuf, 0, sizeof(readBuf) );
	// Dummy Write WORD_Address
	io_write(I2C_0_io, WORD_address, 1);
	// Sequential Read
	io_read(I2C_0_io, readBuf, 16);
	
	printf("EEPROM Data Write/Read Test : %d \n\r", The_Counter) ;

	printf(" - %02X %02X %02X %02X %02X %02X %02X %02X\r\n",
	readBuf[0], readBuf[1], readBuf[2], readBuf[3],
	readBuf[4], readBuf[5], readBuf[6], readBuf[7] );
	printf(" - %02X %02X %02X %02X %02X %02X %02X %02X\r\n\n",
	readBuf[8], readBuf[9], readBuf[10], readBuf[11],
	readBuf[12], readBuf[13], readBuf[14], readBuf[15] );
}



/**
 * \brief Callback to get the Wi-Fi status update.
 *
 * \param[in] u8MsgType type of Wi-Fi notification. Possible types are:
 *  - [M2M_WIFI_RESP_CON_STATE_CHANGED](@ref M2M_WIFI_RESP_CON_STATE_CHANGED)
 *  - [M2M_WIFI_REQ_DHCP_CONF](@ref M2M_WIFI_REQ_DHCP_CONF)
 * \param[in] pvMsg A pointer to a buffer containing the notification parameters
 * (if any). It should be casted to the correct data type corresponding to the
 * notification type.
 */
static void wifi_cb(uint8_t u8MsgType, void *pvMsg)
{
	switch (u8MsgType) {
	case M2M_WIFI_RESP_CON_STATE_CHANGED: {
		tstrM2mWifiStateChanged *pstrWifiState = (tstrM2mWifiStateChanged *)pvMsg;
		if (pstrWifiState->u8CurrState == M2M_WIFI_CONNECTED) {
		} else if (pstrWifiState->u8CurrState == M2M_WIFI_DISCONNECTED) {
			printf("Station disconnected\r\n");
			DisConnection_Beep();
			
		}

		break;
	}

	case M2M_WIFI_REQ_DHCP_CONF: {
		uint8_t *pu8IPAddress = (uint8_t *)pvMsg;
		printf("Station connected\r\n");
		printf("Station IP is %u.%u.%u.%u\r\n", pu8IPAddress[0], pu8IPAddress[1], pu8IPAddress[2], pu8IPAddress[3]);
		// Beep 3 times to indicate the connection
		Connection_Beep();
		break;
	}

	default: {
		break;
	}
	}
}


unsigned int	ADC_Read_sync(void)
{
	uint8_t readBuf[2];

		// AIN6 -- (PB04)

		adc_sync_enable_channel(&ADC_0, 0);
		adc_sync_set_inputs(&ADC_0, ADC_INPUTCTRL_MUXPOS_AIN6_Val, ADC_INPUTCTRL_MUXNEG_GND_Val, 0);

		adc_sync_read_channel(&ADC_0, 0, readBuf, 2);
		return (readBuf[1]* 256 + readBuf[0]) ;
}



/**
 * \brief Main application function.
 *
 * \return program return value.
 */
int main(void)
{
	tstrWifiInitParam param;
	tstrM2MAPConfig   strM2MAPConfig;
	int8_t            ret = 0;

	atmel_start_init();
	TIMER_0_Initial() ;
	I2C_0_Initial() ;
	LCM_Init();
	PowerOn_Beep() ;
	
	LCM_SetCursor(0,0);
	LCM_PutROMString(" APP-MASTERS18");
	LCM_SetCursor(9,1);
	LCM_PutROMString("VR=");	
		LCM_SetCursor(0,1);
		LCM_PutROMString("TMP=");	
	

	/* Initialize Wi-Fi parameters structure. */
	memset((uint8_t *)&param, 0, sizeof(tstrWifiInitParam));

	/* Initialize Wi-Fi driver with data and status callbacks. */
	param.pfAppWifiCb = wifi_cb;
	wifi_init(&param);

	/* Initialize AP mode parameters structure with SSID, channel and OPEN security type. */
	memset(&strM2MAPConfig, 0x00, sizeof(tstrM2MAPConfig));
	strcpy((char *)&strM2MAPConfig.au8SSID, MAIN_WLAN_SSID);
	strM2MAPConfig.u8ListenChannel = MAIN_WLAN_CHANNEL;
	strM2MAPConfig.u8SecType       = MAIN_WLAN_AUTH;

	strM2MAPConfig.au8DHCPServerIP[0] = 192;
	strM2MAPConfig.au8DHCPServerIP[1] = 168;
	strM2MAPConfig.au8DHCPServerIP[2] = 1;
	strM2MAPConfig.au8DHCPServerIP[3] = 1;

#if USE_WEP
	strcpy((char *)&strM2MAPConfig.au8WepKey, MAIN_WLAN_WEP_KEY);
	strM2MAPConfig.u8KeySz   = strlen(MAIN_WLAN_WEP_KEY);
	strM2MAPConfig.u8KeyIndx = MAIN_WLAN_WEP_KEY_INDEX;
#endif

	/* Bring up AP mode with parameters structure. */
	ret = m2m_wifi_enable_ap(&strM2MAPConfig);
	if (M2M_SUCCESS != ret) {
		printf("main: m2m_wifi_enable_ap call error!\r\n");
		while (1) {
		}
	}

	printf("AP mode started. You can connect to %s.\r\n", (char *)MAIN_WLAN_SSID);
	
	Error_Code = 0 ;
	Board_Test();

	while (1) {
		/* Handle pending events from network controller. */
 		while (m2m_wifi_handle_events(NULL) != M2M_SUCCESS) {
		}
		
		if ( Report_Temp == 1)
		{
			VR1_Value = ADC_Read_sync() ;
			
			LCM_SetCursor(12,1);
			LCM_PutNumber(VR1_Value,4);
			
			LCM_SetCursor(4,1);
			LCM_PutNumber(TempValue,3);
						
			MCP9808_Read();

			if(gpio_get_pin_level(SW_S1) == false)
			{
					printf("ADC = %d \n\n\r",VR1_Value) ;
					printf("MCP9808 Read Test : %d \n\r", The_Counter ) ;
					printf(" --->  %d Degree C \r\n\n", TempValue ) ;
			}
			if(gpio_get_pin_level(SW_S2) == false)
			AT24MAC402_WR_Test() ;
			Report_Temp = 0 ;
			The_Counter ++ ;
			
			LED_Speed = LED_Speed_Data[VR1_Value/256] ;
			
		}

		
	}

	return 0;
}

void	PowerOn_Beep(void)
{
	Buzzer_Action = 1 ;
	Buzzer_State = 1 ;
	Buzzer_Action_Data[0] = 20 ;
	Buzzer_Action_Data[1] = 20 ;
	Buzzer_Action_Data[2] = 20 ;
	Buzzer_Action_Data[3] = 20 ;	
	Buzzer_Action_Data[4] = 999 ;	
}

void	Connection_Beep(void)
{
	Buzzer_Action = 1 ;
	Buzzer_State = 1 ;
	Buzzer_Action_Data[0] = 60 ;
	Buzzer_Action_Data[1] = 20 ;
	Buzzer_Action_Data[2] = 20 ;
	Buzzer_Action_Data[3] = 20 ;
	Buzzer_Action_Data[4] = 20 ;
	Buzzer_Action_Data[5] = 20 ;
	Buzzer_Action_Data[6] = 999 ;
}

void	DisConnection_Beep(void)
{
	Buzzer_Action = 1 ;
	Buzzer_State = 1 ;
	Buzzer_Action_Data[0] = 10 ;
	Buzzer_Action_Data[1] = 10 ;
	Buzzer_Action_Data[2] = 10 ;
	Buzzer_Action_Data[3] = 10 ;
	Buzzer_Action_Data[4] = 10 ;
	Buzzer_Action_Data[5] = 10 ;
	Buzzer_Action_Data[6] = 10 ;
	Buzzer_Action_Data[7] = 10 ;
	Buzzer_Action_Data[8] = 999 ;
}


void	Board_Test(void)
{

	uint8_t Test_readBuf[16];
	uint8_t Test_WORD_address[1] = { 0x0 };
	uint8_t Test_DATA_bytes[17] = { 0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF,0x10};
	unsigned char Test_TempValue ;

	// Read Temperature , Set Error Code as 2 if Temperature read failed 

	// Set I2C address to MCP9808 , A2/A1/A0 = 111, address = 0011 111 = 0x1F
	i2c_m_sync_set_slaveaddr(&I2C_0, 0x1F, I2C_M_SEVEN);

	// Random Read DATA_bytes
	memset( Test_readBuf, 0, sizeof(Test_readBuf) );
	// Write Temperature Read Command : Address 0110
	Test_WORD_address[0] = 0x05 ;
	io_write(I2C_0_io, Test_WORD_address, 1);
	// Sequential Read
	io_read(I2C_0_io, Test_readBuf, 2);
	
	Test_readBuf[0] &= 0x0f ;
	Test_TempValue = ( (unsigned int)Test_readBuf[0] * 256 + Test_readBuf[1] ) * 0.0625 ;

	if ((Test_TempValue < 5 ) || (Test_TempValue > 50))
	{
		Error_Code = 2 ;
		return ;
	}
	
	else
		printf ("MCP9808 Test OK ! \n\n\r") ;


	Test_WORD_address[0] = 0x0 ;
	// Set I2C address to AT24MAC402 , A2/A1/A0 = 110, address = 1010 110 = 0x56
	i2c_m_sync_set_slaveaddr(&I2C_0, 0x56, I2C_M_SEVEN);

	// Page Write DATA_bytes, first byte is WORD_address
	io_write(I2C_0_io, Test_DATA_bytes, 17);

	// Write Cycle Time >5ms
	delay_ms(5);

	// Random Read DATA_bytes
	memset( Test_readBuf, 0, sizeof(Test_readBuf) );
	// Dummy Write WORD_Address
	io_write(I2C_0_io, Test_WORD_address, 1);
	// Sequential Read
	io_read(I2C_0_io, Test_readBuf, 16);
	
	if ((Test_readBuf[0] != Test_DATA_bytes[1]) || (Test_readBuf[7] != Test_DATA_bytes[8]))
	Error_Code = 3 ;
	else
		printf ("AT24MAC402 Test OK ! \n\n\r") ;
	
}
